/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSHANDLEBOX_H
#define PAPYRUSHANDLEBOX_H

#include <papyrus/boxed.h>

namespace Papyrus {

  /**
    @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
  */
  class Handlebox : public Boxed {
    public:

      typedef PapyrusSmartPointer<Handlebox> pointer;

      Handlebox(Handles handles=CORNER_HANDLES);

      static Handlebox::pointer create(Handles handles=CORNER_HANDLES);

      virtual ~Handlebox();

      void set_handle_shape( Position p, Drawable::pointer shape, bool activate=true );

      void set_handle_shape( Handles h, Drawable::pointer shape, bool activate=true );

      Drawable::pointer get_handle_shape( Position p );

      void activate_handle( Position p, bool activate=true );

      void activate_handle( Handles h, bool activate=true );

      void deactivate_handle( Position p );

      void deactivate_handle( Handles h );

      bool is_active( Position p );

      bool is_active( Handles h );

      int inside_handle( double x, double y );

      PAPYRUS_CLASS_NAME("Handle Box");

      PAPYRUS_CLONE_METHOD( Handlebox );

    protected:
      Drawable::pointer m_handle_shape[ 8 ];
      bool m_handle_active[8];
      bool m_initializing;

      /**
       * Reimplements Boxed' behavior when extents need to be updated.
       * Performs same behavior, but also adjusts handle shapes to their proper places.
       */
      virtual void rebuild_extents();

      virtual void position_handle( Drawable::pointer, double x, double y );

      virtual void on_child_added( Drawable::pointer child );

      virtual void on_child_removed( Drawable::pointer child );

  };

}

#endif
