/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSLINESTYLE_H
#define PAPYRUSLINESTYLE_H

#include <cmath>
#include <papyrus/renderable.h>
#include <papyrus/drawable.h>

namespace Papyrus
{

  /**
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
  class LineStyle : public Renderable
  {
    public:
      LineStyle( Cairo::RefPtr<Cairo::Pattern> pattern = Cairo::RefPtr<Cairo::Pattern>(),
                 double width = 1.0
               );

      LineStyle( double width,
                 Cairo::LineCap cap = Cairo::LINE_CAP_BUTT,
                 Cairo::LineJoin join = Cairo::LINE_JOIN_MITER
               );

      LineStyle( Cairo::RefPtr<Cairo::Pattern> pattern,
                 double width,
                 Cairo::LineCap cap,
                 Cairo::LineJoin join
               );

      ~LineStyle();

      LineStyle& operator=( const LineStyle& other );

      bool operator==( const LineStyle& other ) const;

      bool operator!=( const LineStyle& other ) const;

      const std::valarray< double >& dash_array();

      double dash_offset();

      void set_dash ( const std::valarray< double > dashes = std::valarray<double>(), double offset = 0.0 );

      Cairo::LineCap cap();

      void set_cap ( Cairo::LineCap cap = Cairo::LINE_CAP_BUTT );

      Cairo::LineJoin join();

      void set_join ( Cairo::LineJoin join = Cairo::LINE_JOIN_MITER );

      double width();

      void set_width ( double width = 1.0 );

      double miter_limit();

      void set_miter_limit ( double limit = INFINITY );

      Cairo::RefPtr<Cairo::Pattern> pattern();

      void set_pattern( Cairo::RefPtr<Cairo::Pattern> pattern = Cairo::RefPtr<Cairo::Pattern>() );

      virtual void render( Cairo::RefPtr<Cairo::Context> cairo );

      sigc::signal<void> signal_pattern();
      sigc::signal<void> signal_width();
      sigc::signal<void> signal_cap();
      sigc::signal<void> signal_join();
      sigc::signal<void> signal_miter_limit();
      sigc::signal<void> signal_dash();

    protected:
      Cairo::RefPtr<Cairo::Pattern> m_pattern;
      double m_width;
      Cairo::LineCap m_cap;
      Cairo::LineJoin m_join;
      double m_miter_limit;
      std::valarray<double> m_dash_array;
      double m_dash_offset;

      sigc::signal<void> m_signal_pattern;
      sigc::signal<void> m_signal_width;
      sigc::signal<void> m_signal_cap;
      sigc::signal<void> m_signal_join;
      sigc::signal<void> m_signal_miter_limit;
      sigc::signal<void> m_signal_dash;

  };

}

#endif
